/*
                                 mondesi.js

Relies heavily on functionality provided by WZ_JSGRAPHICS.JS (thanks Walter).

Plotting routines assume a 300w x 100h graph (yep, q&d).


COPYRIGHT
---------
Copyright (C) 2008-2013 Mark G.Daniel
This program, comes with ABSOLUTELY NO WARRANTY.
This is free software, and you are welcome to redistribute it under the
conditions of the GNU GENERAL PUBLIC LICENSE, version 3, or any later version.
http://www.gnu.org/licenses/gpl.txt


VERSION
-------
06-JAN-2013  MGD  refinement (perhaps rework is closer)
13-OCT-2010  MGD  refinement
08-JUL-2010  MGD  add WebSocket support
29-NOV-2008  MGD  initial
*/

// versions of MONDESI.EXE this JavaScript is compatible with
var mdsCompatibleVersions = new Array ("3.0.0"); 

// note: many storage items are instantiated by MONDESI.C

var mdsBarCol1 = "LimeGreen",
    mdsBarCol2 = "Chocolate",
    mdsBarColAve = "Yellow",
    mdsHistCol1 = "DarkGreen",
    mdsHistCol2 = "Sienna",
    mdsHistColAve = "GoldenRod",
    mdsObviousBtn = "Gold";

var mdsNodeData = new Array();

var mdsAlertSecs = 90,  // ninety seconds
    mdsClusterVisible = false,
    mdsDiskVisible = false,
    mdsPauseInProgress = false,
    mdsPauseStopTimer = null,
    mdsPauseStopSecs = 300,   // five minutes
    mdsPrintInProgress = false,
    mdsPrintTimeout = 30,  // 30 seconds
    mdsProcVisible = false,
    mdsRunning = false,
    mdsScaleTimer = null,
    mdsScaleSecs = 300,  // five minutes
    mdsUpdatePause = false,
    mdsTimeStamp = 0,
    mdsUpdateDisplay = false,
    mdsVersionNBG = false,
    mdsWzPrintable = false;

var noteAlertSpan = '<span style="font-size:140%;font-weight:bold;' +
                    'margin:5px;float:right;color:red;">';
var mdsAlertEndSpan = '</span>';

var mdsAlertText = noteAlertSpan + 'STALLED' + mdsAlertEndSpan;

var mdsPauseStopText = noteAlertSpan + 'STOPPED' + mdsAlertEndSpan;

var mdsProcTable = '<table id="proc_table" class="proc_table">' +
                   '<tr><th>CPU</th><th>S+E+K</th><th>FLTS</th><th>BIO</th>' +
                   '<th>DIO</th></tr>';
var mdsProcEndTable = '</table>';

var incompatVersion = 'JavaScript and executable are incompatible!\n' +
                      'Reload and/or clear browser cache.';

if (mdsCompatibleVersions.indexOf(mdsExeVersion) == -1) mdsVersionNBG = true;

// initial start
// -------------

function mdsInitialStart ()
{
   if (mdsVersionNBG)
   {
      alert(incompatVersion);
      return;
   }
   mdsStreamData (true);
   mdsHistoryClick (false);
   mdsScaleTimer = setTimeout("mdsScale()",mdsScaleSecs*1000);
   mdsRunning = mdsUpdateDisplay = true;
}

// update the textual node information
// -----------------------------------

function mdsNodeNotes (text)
{
   if (!mdsUpdateDisplay) return;

   var nodeNotes = document.getElementById("node_notes");
   nodeNotes.innerHTML = null;
   nodeNotes.innerHTML = text;
   // each minute the alert gets recheduled if updating is occuring
   mdsResetAlert(true);
}

// (re)initialise per-node data
// ----------------------------

function mdsNodeInit (which,active)
{
   var which1 = "node_" + which;
   if (mdsNodeData[which1] == undefined)
   {
      mdsNodeData[which1] = new Object();
      mdsNodeData[which1].timer = null;
      mdsNodeData[which1].which = which;
      mdsNodeData[which1].title = which.toUpperCase();
      mdsNodeData[which1].active = active;
      mdsNodeData[which1].graphic = new jsGraphics(which1);
   }

   mdsNodeData[which1].ave = mdsNodeData[which1].max =
      mdsNodeData[which1].val1 = mdsNodeData[which1].val2 =
      mdsNodeData[which1].scale = 0;
   mdsNodeData[which1].fixed = false;

   mdsBarGraph (which);
   mdsBarPaint (which);

   mdsHistInit (which);
   mdsHistGraph (which);
   mdsHistPaint (which);
}

// set and repaint the title
// -------------------------

function mdsNodeTitle (which,title)
{
   mdsNodeData["node_"+which].title = title;
   mdsMonVal(which);
}

// scale displays according to maximum datum
// -----------------------------------------

function mdsScale ()
{
   for (which1 in mdsNodeData)
   {
      if (mdsNodeData[which1].fixed) continue;
      var which = which1.substring(5);
      var data1 = mdsHistData["hist_"+which].data1;
      var data2 = mdsHistData["hist_"+which].data2;

      var max = 0;
      for (tdata in data1)
         if (data1[tdata] > max) max = data1[tdata];
      mdsNodeData[which1].max = mdsNodeData[which1].scale = max;
      mdsNodeData[which1].ave = 0;
      mdsBarPlot(which);
      mdsHistPlot(which);
   }

   clearTimeout(mdsScaleTimer);
   mdsScaleTimer = setTimeout("mdsScale()",mdsScaleSecs*1000);
}

// zero node and history data, reset graphs
// ----------------------------------------

function mdsNodeZero ()
{
   for (which1 in mdsNodeData)
   {
      var which = which1.substring(5);
      mdsNodeInit(which);
      mdsMonVal(which,0);
   }
}

// record a datum and graph its value
// ----------------------------------
// (called by the data streaming in the MONDESI hidden iframe)

function mdsMonVal (which,scale,value1,value2)
{
   var which1 = "node_" + which;
   if (mdsNodeData[which1] == undefined) return;

   if (!mdsNodeData[which1].active) return;

   if (scale == undefined) scale = -1;
   if (value1 == undefined) value1 = -1;
   if (value2 == undefined) value2 = -1;

   // arrange a timed update (must be done before 'scale' is adjusted)
   var updateGraph = 'mdsMonVal("'+which+'",'+scale+','+value1+','+value2+')';
   clearTimeout(mdsNodeData[which1].timer);
   mdsNodeData[which1].timer = setTimeout(updateGraph,mdsRefreshInterval*2000);

   if (value1 > mdsNodeData[which1].max) mdsNodeData[which1].max = value1;
   var max = mdsNodeData[which1].max;
   if (scale >= 0)
      mdsNodeData[which1].fixed = true;
   else
      scale = max;
   mdsNodeData[which1].scale = scale;

   var average = 0;
   if (which != "mem" && which != "pgfl" && which != "prc")
   {
      var data1 = mdsHistData["hist_"+which].data1;
      var count = 0;
      var total = 0;
      for (tdata in data1)
      {
         count++;
         total += data1[tdata];
      }
      if (count > 10) average = Math.round((total / count));
   }

   mdsNodeData[which1].val1 = value1;
   mdsNodeData[which1].val2 = value2;
   mdsNodeData[which1].ave = average;

   mdsBarPlot(which);

   // history data
   mdsHistVal (which,value1,value2)
}

// set the process table
// ---------------------

function mdsMonProc (table)
{
   if (!mdsUpdateDisplay) return;

   var procTableDiv = document.getElementById("proc_table_div");
   procTableDiv.innerHTML = null;
   procTableDiv.innerHTML = mdsProcTable + table + mdsProcEndTable;
}

// cancel any outstanding alert and optionally set a new one
// ---------------------------------------------------------

var mdsAlertTimer = null;

function mdsResetAlert (newAlert)
{
   // each minute the alert gets recheduled if updating is occuring
   clearTimeout(mdsAlertTimer);
   mdsAlertTimer = null;
   if (newAlert) mdsAlertTimer = setTimeout("mdsAlert()",mdsAlertSecs*1000);
}

// that the node has stopped being updated
// ---------------------------------------

function mdsAlert ()
{
   // the above timer has expired
   mdsAlertTimer = null;
   var nodeNotes = document.getElementById("node_notes");
   var notesText = nodeNotes.innerHTML;
   nodeNotes.innerHTML = null;
   nodeNotes.innerHTML = mdsAlertText + notesText;
}

// send a message from a child iframe to the parent
// ------------------------------------------------

function mdsToMsgChannel (forEval)
{
   // MSIE(6) and SeaMonkey(1.6) (at least) deny permission for these
   try
   {
      var win = parent.window;
      if (win != undefined && typeof win.postMessage == "function")
         // message posting browsers (HTML5)
         win.postMessage (forEval, "*");
      else
         // inter-frame kludge
         mdsToBackChannel (forEval);
   }
   catch (err)
   {
      // inter-frame kludge
      mdsToBackChannel (forEval);
   }
}

// message passing via hidden iframe (HTML4)
// -----------------------------------------

var mdsBackChanCount = 0;

function mdsToBackChannel (forEval)
{
   var url = mdsRefererHost + "/mondesi/-/js/bchan.html#" + forEval;
   // MSIE(6+7) gets confused removing then adding elements with identical IDs!
   var iframe = document.getElementById("bchan_iframe_"+mdsBackChanCount);
   if (iframe != undefined)
   {
      mdsBody.removeChild(iframe);
      delete iframe;
   }
   iframe = document.createElement("iframe");
   iframe.setAttribute("id","bchan_iframe_"+(mdsBackChanCount++));
   iframe.setAttribute("src",url);
   iframe.setAttribute("height",0);
   iframe.setAttribute("width",0);
   iframe.setAttribute("frameBorder",0);
   mdsBody.appendChild(iframe);
}

// stop/start the streaming data flow
// ----------------------------------

var mdsDataIframeCount = 0;
var mdsDataIframe = undefined;

function mdsStreamData (start)
{
   if (WASDwebSocket)
   {
      // close any existing connections
      webSocketClose();
      mdsResetAlert (false);
      if (!start) return;

      if (webSocketOpen())
      {
         mdsResetAlert (true);
         return;
      }
      // it's entirely possible the server but not browser supports WebSocket
      WASDwebSocket = false;
   }

   // via iframe and HTML push

   if (mdsDataIframe != undefined)
   {
      // MSIE (6+7) seems to need this to dissociate from the data stream
      mdsDataIframe.src = "/mondesi/-/js/nl.html";

      /* dispose of the iframe dropping the data stream */
      mdsBody.removeChild(mdsDataIframe);
      delete mdsDataIframe;
      mdsDataIframe = undefined;
      mdsResetAlert (false);
   }

   if (start)
   {
      var url = mdsScriptName + "?" + mdsRefreshInterval;
      mdsDataIframe = document.createElement("iframe");
      // MSIE(6+7) gets confused removing then adding elements with same IDs!
      mdsDataIframe.setAttribute("id","node_iframe_"+mdsDataIframeCount);
      mdsDataIframe.setAttribute("src",url);
      mdsDataIframe.setAttribute("height",0);
      mdsDataIframe.setAttribute("width",0);
      mdsDataIframe.setAttribute("frameBorder",0);
      mdsBody.appendChild(mdsDataIframe);
      mdsResetAlert (true);
   }
}

// cluster button has been toggled
// -------------------------------

function mdsClusterClick (setValue)
{
   var nodeScs = document.getElementById("node_scs");
   var histScs = document.getElementById("hist_scs");
   if (mdsCluster && setValue == undefined)
      mdsClusterVisible = !mdsClusterVisible;
   if (mdsClusterVisible) {
      nodeScs.style.display = "inline";
      // better as "table-row" but <MSIE8 won't accept it
      histScs.style.display = "";
   }
   else
      nodeScs.style.display = histScs.style.display = "none";
   mdsHistoryClick (mdsHistVisible);
   return true;
}

// disk button has been toggled
// ----------------------------

function mdsDiskClick (setValue)
{
   var nodeDisk = document.getElementById("node_disk_device");
   if (mdsDiskCount > 0 && setValue == undefined)
      mdsDiskVisible = !mdsDiskVisible;
   if (mdsDiskVisible)
      nodeDisk.style.display = "inline";
   else
      nodeDisk.style.display = "none";
   mdsHistoryClick (mdsHistVisible);
   return true;
}

// proc button has been toggled
// ----------------------------

function mdsProcClick (setValue)
{
   var procDiv = document.getElementById("proc_wrapt");
   var procTableDiv = document.getElementById("proc_table_div");
   if (mdsProcCount > 0 && setValue == undefined)
      mdsProcVisible = !mdsProcVisible;
   if (mdsProcVisible) {
      procDiv.style.display = "block";
      procTableDiv.innerHTML = mdsProcTable + mdsProcEndTable;
   }
   else {
      procDiv.style.display = "none";
      procTableDiv.innerHTML = null;
   }
   mdsHistoryClick (mdsHistVisible);
   webSocketProcMon ();
   return true;
}

// history button has been toggled
// -------------------------------

function mdsHistoryClick (setValue)
{
   var nodeHeight = (mdsBarHeight * 9) + 20;
   var histHeight = (mdsGraphHeight * 3) + 34;
   var procHeight = 0;

   if (mdsCluster) {
      var scsBtn = document.getElementById("scs_btn");
      scsBtn.disabled = false;
      if (mdsClusterVisible) {
         nodeHeight += mdsBarHeight * 3;
         histHeight += mdsGraphHeight;
         histHeight += 10;
      }
   }

   if (mdsDiskCount > 0) {
      var diskBtn = document.getElementById("disk_btn");
      diskBtn.disabled = false;
      if (mdsDiskVisible) {
         nodeHeight += mdsBarHeight * mdsDiskCount;
         histHeight += mdsGraphHeight * Math.floor(mdsDiskCount / 3);
         if (mdsDiskCount % 3) histHeight += mdsGraphHeight;
         histHeight += 10 *Math.floor(mdsDiskCount / 3);
         if (mdsDiskCount % 3) histHeight += 10;
         histHeight += 3;
      }
   }

   if (mdsProcCount > 0) {
      var procBtn = document.getElementById("proc_btn");
      procBtn.disabled = false;
      if (mdsProcVisible) procHeight = mdsProcHeight;
   }

   var histGraph = document.getElementById("history_graph");
   if (setValue == undefined) mdsHistVisible = !mdsHistVisible;
   if (mdsHistVisible) {
      var height = nodeHeight + procHeight + histHeight;
      histGraph.style.display = "inline";
   }
   else {
      var height = nodeHeight + procHeight;
      histGraph.style.display = "none";
   }
   var remotePC = "mdsIframeHeight('" + mdsIframeId + "'," +
                  mdsHistVisible + "," + height + ")";
   mdsToMsgChannel(remotePC);
   return true;
}

// pause and release the display update
// ------------------------------------

var mdsPauseBgCol;

function mdsPauseClick ()
{ 
   var btn = document.getElementById("pause_btn");
   if (mdsRunning && !mdsPauseInProgress)
   {
      // pause the data updates
      mdsPauseBgCol = btn.style.backgroundColor;
      btn.style.backgroundColor = mdsObviousBtn;
      btn.value = "release";
      mdsResetAlert(false);
      clearTimeout(mdsScaleTimer);
      mdsScaleTimer = null;
      mdsPauseStopTimer = setTimeout("mdsPauseStop()",mdsPauseStopSecs*1000);
      // just in case the browser print dialogue is engaged whilst stopped
      mdsMakePrintable(true);
      mdsPauseInProgress = true;
      mdsUpdateDisplay = false;
   }
   else
   if (mdsRunning)
   {
      // recommence the data updates
      btn.style.backgroundColor = mdsPauseBgCol;
      btn.value = "pause";
      mdsResetAlert(true);
      mdsMakePrintable(false);
      mdsProcClick(mdsProcVisible);
      clearTimeout(mdsPauseStopTimer);
      mdsPauseStopTimer = null;
      mdsScaleTimer = setTimeout("mdsScale()",mdsScaleSecs*1000);
      mdsPauseInProgress = false;
      if (!mdsPrintInProgress) mdsUpdateDisplay = true;
   }
   else
   {
      // restart data collection and display
      btn.style.backgroundColor = mdsPauseBgCol;
      btn.value = "pause";
      mdsStreamData (true);
      mdsNodeZero();
      if (mdsPrintInProgress) 
         mdsPrintClick();
      else
         mdsMakePrintable(false);
      mdsProcClick(mdsProcVisible);
      mdsScaleTimer = setTimeout("mdsScale()",mdsScaleSecs*1000);
      mdsPauseInProgress = false;
      mdsRunning = mdsUpdateDisplay = true;
   }
   return true;
}

// timer-driven stop
// -----------------

function mdsPauseStop ()
{
   // been paused for long enough!
   mdsRunning = mdsUpdateDisplay = false;
   mdsPauseStopTimer = null;
   clearTimeout(mdsScaleTimer);
   mdsScaleTimer = null;
   // shut down the client/server IPC
   mdsStreamData (false);
   mdsPauseInProgress = false;
   var btn = document.getElementById("pause_btn");
   btn.value = "START";
   // append an informational to the notes
   var nodeNotes = document.getElementById("node_notes");
   var notesText = nodeNotes.innerHTML;
   nodeNotes.innerHTML = null;
   nodeNotes.innerHTML = mdsPauseStopText + notesText;
}

// make the first visible the second not
// -------------------------------------

function mdsForeFront (which1,which2)
{
   document.getElementById(which1).style.display = "block";
   document.getElementById(which2).style.display = "none";
}

// printing - a juggling act
// -------------------------

var mdsPrintBgCol;
var mdsPrintClickTimer = null;

function mdsPrintClick ()
{
   if (mdsPrintInProgress)
      mdsAfterPrint();
   else
   {
      mdsBeforePrint();
      setTimeout("window.print()",100);
   }
   return true;
}

function mdsBeforePrint ()
{
   if (mdsPrintInProgress) return true;
   mdsPrintInProgress = true;
   var btn = document.getElementById("print_btn");
   mdsPrintBgCol = btn.style.backgroundColor;
   btn.style.backgroundColor = mdsObviousBtn;
   clearTimeout(mdsPrintClickTimer);
   mdsPrintClickTimer = null;
   mdsPrintClickTimer = setTimeout("mdsPrintClick()",mdsPrintTimeout*1000);
   mdsResetAlert(false);
   mdsMakePrintable(true);
   mdsUpdateDisplay = false;
   return true;
};

function mdsAfterPrint()
{
   if (!mdsPrintInProgress) return true;
   mdsPrintInProgress = false;
   var btn = document.getElementById("print_btn");
   btn.style.backgroundColor = mdsPrintBgCol;
   clearTimeout(mdsPrintClickTimer);
   mdsPrintClickTimer = null;
   mdsResetAlert(true);
   mdsMakePrintable(false);
   if (!mdsPauseInProgress) mdsUpdateDisplay = true;
   return true;
};

// this is in-line code
if (window.matchMedia)
{
    var mediaQueryList = window.matchMedia('print');
    mediaQueryList.addListener(function(mql) {
        if (mql.matches)
           mdsBeforePrint();
        else
           mdsAfterPrint();
    });
}
else
{
   // a la MSIE
   window.onbeforeprint = mdsBeforePrint;
   window.onafterprint = mdsAfterPrint;
}

// (re)set WZ printable flag and redraw
// ------------------------------------

function mdsMakePrintable (MakePrint)
{
   mdsWzPrintable = MakePrint;
   for (which1 in mdsNodeData)
   {
      var which = which1.substring(5);
      mdsBarPlot(which);
      mdsHistPlot(which);
   }
   mdsWzPrintable = false;
}

// return a string with the value as unit, kilo or mega
// ----------------------------------------------------

function mdsValueString (value)
{
   if (value > 999999999) {
      value = (value / 1073741824).toFixed(1);
      if (value > 99) value = (value/1).toFixed(0);
      var dvalue = value.toString() + "G";
   }
   else
   if (value > 999999) {
      value = (value / 1048576).toFixed(1);
      if (value > 99) value = (value/1).toFixed(0);
      var dvalue = value.toString() + "M";
   }
   else
   if (value > 9999) {
      value = (value / 1024).toFixed(1);
      if (value > 99) value = (value/1).toFixed(0);
      var dvalue = value.toString() + "k";
   }
   else
      var dvalue = value.toString();
   return dvalue;
}

// display a numeric value to the right of a bar or graph
// ------------------------------------------------------

function mdsDrawValue (jsgr,which,xpos,ypos,color,value)
{
   var dval = mdsValueString(value);
   xpos = xpos - 5 - (dval.length * 8) + (dval.length * 2);
   if (dval.indexOf("k") >= 0) xpos += 2;
   if (dval.indexOf(".") >= 0) xpos += 2;
   jsgr.setColor(color);
   jsgr.drawString(dval,xpos,ypos);
}

// initialise a single datum bar graph
// -----------------------------------

function mdsBarGraph (which)
{
   var which1 = "node_" + which;
   var jsgr = mdsNodeData[which1].graphic;
   jsgr.recreate();
   jsgr.setPrintable(mdsWzPrintable);
   jsgr.setFont ("arial","10px",Font.PLAIN); 
   if (mdsNodeData[which1].active)
      jsgr.setColor("white");
   else
      jsgr.setColor("#d3d3d3"); // "lightGray" - stupid MSIE
   jsgr.fillRect (0, 0, mdsBarWidth+2, mdsBarHeight);
}

// add the axes and title of a bar graph then paint
// ------------------------------------------------

function mdsBarPaint (which)
{
   var which1 = "node_" + which;
   var jsgr = mdsNodeData[which1].graphic;
   jsgr.setColor("black");
   jsgr.drawRect (0, 0, mdsBarWidth+2, mdsBarHeight);
   if (which == "cpu" && mdsCpuCount > 1)
      jsgr.drawString(mdsNodeData["node_"+which].title+"x"+mdsCpuCount,7,2);
   else
      jsgr.drawString(mdsNodeData[which1].title,7,2);
   jsgr.paint();
}

// render a single datum of the bar graph
// --------------------------------------

function mdsBarPlot (which)
{
   if (!mdsUpdateDisplay) return;

   var which1 = "node_" + which;
   if (mdsNodeData[which1] == undefined) return;

   mdsBarGraph(which);
   if (!mdsNodeData[which1].active)
   {
      mdsBarPaint(which);
      return;
   }

   var height = mdsBarHeight;

   var jsgr = mdsNodeData[which1].graphic;
   jsgr.setColor("white");
   jsgr.fillRect (0, 0, mdsBarWidth+2, mdsBarHeight+2);

   mdsDrawValue(jsgr,which,mdsBarWidth,2,"black",mdsNodeData[which1].scale);

   var widthAve = mdsNodeData[which1].ave *
                  mdsBarWidth / mdsNodeData[which1].scale;
   if (widthAve > mdsBarWidth) widthAve = mdsBarWidth;
   var widthVal1 = mdsNodeData[which1].val1 *
                   mdsBarWidth / mdsNodeData[which1].scale;
   if (widthVal1 > mdsBarWidth) widthVal1 = mdsBarWidth;

   if (mdsNodeData[which1].val2 >= 0)
      var widthVal2 = mdsNodeData[which1].val2 *
                      mdsBarWidth / mdsNodeData[which1].scale;
   else
      var widthVal2 = 0;

   if (widthAve > 0 && widthAve > widthVal1 && widthAve > widthVal2)
   {
      jsgr.setColor(mdsBarColAve);
      jsgr.fillRect (1, 0, widthAve+1, height);
      if (widthAve > 60)
         mdsDrawValue(jsgr,which,widthAve,2,"black",mdsNodeData[which1].ave);
   }

   if (widthVal1 > 0 && widthVal1 > widthVal2)
   {
      jsgr.setColor(mdsBarCol1);
      jsgr.fillRect (1, 0, widthVal1+1, height);
      if (widthVal1 > 60)
         mdsDrawValue(jsgr,which,widthVal1,2,"black",mdsNodeData[which1].val1);
   }

   if (widthVal2 > 0)
   {
      jsgr.setColor(mdsBarCol2);
      jsgr.fillRect (1, 0, widthVal2+1, height);
      if (widthVal2 > 60)
         mdsDrawValue(jsgr,which,widthVal2,2,"black",mdsNodeData[which1].val2);
   }

   mdsBarPaint(which);
}

// initialise history data collection and graph rendering
// ------------------------------------------------------

var mdsHistData = new Array();

function mdsHistInit (which)
{
   var which1 = "hist_" + which;
   if (mdsHistData[which1] != undefined)
   {
      var jsgr = mdsHistData[which1].graphic.recreate();
      delete mdsHistData[which1].data1;
      delete mdsHistData[which1].data2;
      delete mdsHistData[which1];
   }
   else
      var jsgr;
   mdsHistData[which1] = new Object();
   mdsHistData[which1].data1 = new Array();
   mdsHistData[which1].data2 = new Array();
   if (jsgr == undefined)
   {
      mdsHistData[which1].graphic = jsgr = new jsGraphics(which1);
      mdsHistData[which1].graphic.setPrintable(mdsWzPrintable);
   }
   else
      mdsHistData[which1].graphic = jsgr;
   jsgr.setPrintable(mdsWzPrintable);
   mdsHistData[which1].visible = false;
}

// record a datum in a sparse (js hash) array
// ------------------------------------------

function mdsHistVal (which,value1,value2)
{
   var which1 = "hist_" + which;
   var data1 = mdsHistData[which1].data1,
       data2 = mdsHistData[which1].data2;

   var tdata = Math.round(mdsTimeStamp).toString();
   if (value1 >= 0) data1[tdata] = value1;
   if (value2 >= 0) data2[tdata] = value2;

   mdsHistPlot (which);
}

// initialise a history line graph
// -------------------------------

function mdsHistGraph (which)
{
   var which1 = "hist_" + which;
   var jsgr = mdsHistData[which1].graphic;
   jsgr.recreate();
   jsgr.setPrintable(mdsWzPrintable);
   jsgr.setFont ("arial","10px",Font.PLAIN); 
   if (mdsNodeData["node_"+which].active)
      jsgr.setColor("white");
   else
      jsgr.setColor("#d3d3d3"); // "lightGray" - stupid MSIE
   jsgr.fillRect (1, 1, mdsGraphWidth+1, mdsGraphHeight+1);
}

// provide the axes and title of the graph then display
// ----------------------------------------------------

function mdsHistPaint (which)
{
   var which1 = "hist_" + which;
   var jsgr = mdsHistData[which1].graphic;
   jsgr.setColor("black");
   jsgr.drawRect (0, 0, mdsGraphWidth+2, mdsGraphHeight+2);
   for (var x = 30; x < mdsGraphWidth-2; x += 30)
   {
      if (x % 60)
         jsgr.drawLine (x+1, mdsGraphHeight+1, x+1, mdsGraphHeight+1-2);
      else
         jsgr.drawLine (x+1, mdsGraphHeight+1, x+1, mdsGraphHeight+1-5);
   }
   if (which == "cpu" && mdsCpuCount > 1)
      jsgr.drawString(mdsNodeData["node_"+which].title+"x"+mdsCpuCount,5,5);
   else
      jsgr.drawString(mdsNodeData["node_"+which].title,5,5);
   jsgr.paint();
}

// display the numeric scale and values on the plot
// ------------------------------------------------

function mdsHistScale (jsgr,which,value1,value2,scale)
{
   if (scale >= 0)
      mdsDrawValue(jsgr,which,mdsGraphWidth,5,"black",scale);
   if (scale > 0 && value1 >= 0)
      mdsDrawValue(jsgr,which,mdsGraphWidth,20,mdsHistCol1,value1);
   if (scale > 0 && value1 > 0 && value2 >= 0)
      mdsDrawValue(jsgr,which,mdsGraphWidth,35,mdsHistCol2,value2);
}

// display an average on the plot
// ------------------------------

function mdsHistAverage (jsgr,value,scale)
{
   if (value <= 0 || value == NaN) return;
   var average = value * mdsGraphHeight / scale;
   if (average <= 0) return;
   if (average > mdsGraphHeight) average = mdsGraphHeight;
   jsgr.setColor(mdsHistColAve);
   jsgr.drawLine (mdsGraphWidth+1, 100-average+1, 1, 100-average+1);
   if (average < 12)
      jsgr.drawString(mdsValueString(value),5,85-average+2);
   else
   if (average < 85)
      jsgr.drawString(mdsValueString(value),5,100-average+2);
   else
      jsgr.drawString(mdsValueString(value),35,100-average+2);
}

// plot a history graph from stored sparse data
// --------------------------------------------

function mdsHistPlot (which)
{
   if (!mdsUpdateDisplay) return;

   var which1 = "hist_" + which;
   if (mdsHistData[which1] == undefined) return;

   if (!mdsHistVisible)
   {
      // obviously do not plot if not visible!
      if (mdsHistData[which1].visible)
      {
         mdsHistData[which1].visible = false;
         mdsHistData[which1].graphic.recreate();
         mdsHistData[which1].graphic.setPrintable(mdsWzPrintable);
      }
      return;
   }
   mdsHistData[which1].visible = true;

   if (!mdsUpdateDisplay) return;

   var data1 = mdsHistData[which1].data1,
       data2 = mdsHistData[which1].data2;

   mdsHistGraph(which);
   if (!mdsNodeData["node_"+which].active)
   {
      mdsHistPaint (which);
      return;
   }

   var jsgr = mdsHistData[which1].graphic;

   // wz canvas begins in the top-left, pen beneath and right of coords
   var scale = mdsNodeData["node_"+which].scale;

   mdsHistAverage(jsgr,mdsNodeData["node_"+which].ave,scale);

   var scale = mdsNodeData["node_"+which].scale;

   var x = y = lastx1 = lasty1 = lastx2 = lasty2 = 0;
   for (tdata in data1)
   {
      x = mdsTimeStamp - parseInt(tdata);

      // eliminate stale data
      if (x > mdsGraphWidth)
      {
         if (x > mdsGraphWidth + 10)
         {
            delete data1[tdata];
            delete data2[tdata];
            continue;
         }
         x = mdsGraphWidth;
      }

      y = data1[tdata] * mdsGraphHeight / scale;
      if (y > mdsGraphHeight) y = mdsGraphHeight;
      jsgr.setColor(mdsHistCol1);
      if (lasty1 > 0)
         jsgr.drawLine (mdsGraphWidth-lastx1, 100-lasty1+1,
                        mdsGraphWidth-x, 100-lasty1+1);
      jsgr.drawLine (mdsGraphWidth-x, 100-lasty1+1,
                     mdsGraphWidth-x, 100-y+1);
      lastx1 = x;
      lasty1 = y;

      if (data2[tdata] != undefined)
      {
         y = data2[tdata] * mdsGraphHeight / scale;
         if (y > mdsGraphHeight) y = mdsGraphHeight;
         jsgr.setColor(mdsHistCol2);
         if (lasty2 > 0)
            jsgr.drawLine (mdsGraphWidth-lastx2, 100-lasty2+1,
                           mdsGraphWidth-x, 100-lasty2+1);
         jsgr.drawLine (mdsGraphWidth-x, 100-lasty2+1,
                        mdsGraphWidth-x, 100-y+1);
         lastx2 = x;
         lasty2 = y;
      }
   }

   if (lasty1 > 0)
   {
      jsgr.setColor(mdsHistCol1);
      jsgr.drawLine (mdsGraphWidth-lastx1+1, 100-lasty1+1,
                     mdsGraphWidth+1, 100-lasty1+1);
   }
   if (lasty2 > 0)
   {
      jsgr.setColor(mdsHistCol2);
      jsgr.drawLine (mdsGraphWidth-lastx2+1, 100-lasty2+1,
                     mdsGraphWidth+1, 100-lasty2+1);
   }

   mdsHistScale(jsgr,which,mdsNodeData["node_"+which].val1,
                           mdsNodeData["node_"+which].val2,
                           mdsNodeData["node_"+which].scale);

   mdsHistPaint (which);
}

// WebSocket support (for WASD v10.1ff)
// ------------------------------------

var ws = null;
var nodeControl = null;

function webSocketOpen () {

   if (!("WebSocket" in window)) return false;

   if (ws) return true;

   if (window.location.protocol == "http:")
      var URL = "ws://";
   else
      var URL = "wss://";
   URL += window.location.host + mdsScriptName + "?" + mdsRefreshInterval;

   ws = new WebSocket(URL);

   ws.onopen = function(evt) {
   };

   ws.onclose = function(evt) {
      if (ws) ws.close();
      ws = null;
      // try to reconnect in a staggered fashion (in case multiple CGIplus)
      var tout = 1500 + Math.floor((Math.random()*1000));
      if (mdsRunning) setTimeout ("webSocketOpen()", tout);
  };

   ws.onmessage = function (evt) { 
      var data = evt.data;
      eval (data);
   };

   return true;
}

function webSocketClose () {

   if (!WASDwebSocket) return;
   if (!("WebSocket" in window)) return;

   if (ws) ws.close();
   ws = null;
}

function webSocketProcMon () {

   if (!ws) return;
   try {
      if (mdsProcVisible)
         ws.send ("procmon=1");
      else
         ws.send ("procmon=0");
   }
   catch (err) {
      // stop/start? WebSocket probably in process of being established
      setTimeout('webSocketProcMon()',1000);
   }
   return false;
}

// fin!
